<?php

namespace Drupal\testsuite\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\testsuite\LoadResourceService;
use Drupal\testsuite\TestMenuService;
use Drupal\testsuite\InitializerService;

/**
 * Configure example settings for this site.
 */
class PHPUnitTestsForm extends ConfigFormBase {

  /**
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'testsuite.phpunit';

  /**
   * @var \Drupal\testsuite\LoadResourceService
   */
  protected $_loadResourceService;

  /**
   * @var \Drupal\testsuite\TestMenuService
   */
  protected $_testMenuService;

  /**
   * @var \Drupal\testsuite\InitializerService
   */
  protected $_initializerService;

  /**
   * @var array The array of files to create Resource with.
   */
  protected $_loadedFiles = [];

  /**
   * @param \Drupal\testsuite\LoadResourceService $loadResourceService
   * @param \Drupal\testsuite\TestMenuService $testMenuService
   */
  public function __construct(
    LoadResourceService $loadResourceService,
    TestMenuService $testMenuService,
    InitializerService $initializerService
  ) {
    $this->_testMenuService = $testMenuService;
    $this->_loadResourceService = $loadResourceService;
    $this->_initializerService = $initializerService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('testsuite.load_resource.service'),
      $container->get('testsuite.test_menu.service'),
      $container->get('testsuite.initializer.service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'testsuite_phpunit_tests_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $area = 'contrib', $module = 'testsuite', $directory = 'Unit') {
    $top_menu = $this->_testMenuService->buildTestsMenu();
    $module_menu = $this->_testMenuService->buildModuleMenu();
    $this->_loadedFiles = $this->_loadResourceService->getResource('form', 'phpunit', $area, $module, $directory);

    $container['submit'] = [
      '#type' => 'submit',
      '#attributes' => array(
        'class' => ['button', 'button--primary', 'test-suite-top-build-button']
      ),
      '#value' => $this->t('Build Report')
    ];

    foreach($this->_loadedFiles as $loadedModule => $statement) {
      $files = $this->_loadResourceService->getFiles($area, $module, $directory);
      // dd($files);
      $statementString = $statement;
      foreach($files as $file) {
        $statementString .= '<br /><a href="'.$file.'" target="_blank">'.$file.'</a>';
      }
      $container[$loadedModule] = [
        '#type' => 'checkbox',
        '#title' => $this->t($statementString),
        '#attributes' => array(
          'id' => $loadedModule,
          'name' => $loadedModule,
        ),
        '#default_value' => TRUE,
      ];
    }

    $container['area'] = [
      '#type' => 'hidden',
      '#value' => $this->t($area)
    ];

    $container['module'] = [
      '#type' => 'hidden',
      '#value' => $this->t($module)
    ];

    $container['directory'] = [
      '#type' => 'hidden',
      '#value' => $this->t($directory)
    ];

    $container['submit_bottom'] = [
      '#type' => 'submit',
      '#attributes' => array(
        'class' => ['button', 'button--primary']
      ),
      '#value' => $this->t('Build Report')
    ];

    $form['container'] = [
      '#type' => 'html_tag',
      '#tag' => 'div',
      '#attributes' => [
        'id' => 'test-suite-ajax-response-container',
        'class' => ['test-suite-container']
      ],
      'children' => [
        $top_menu,
        [
          '#type' => 'html_tag',
          '#tag' => 'div',
          '#attributes' => ['class' => ['test-suite-row']],
          'children' => [
            [
              '#type' => 'html_tag',
              '#tag' => 'div',
              '#attributes' => ['class' => ['test-suite-menu']],
              'child' => $module_menu
            ],
            [
              '#type' => 'html_tag',
              '#tag' => 'div',
              '#attributes' => ['class' => ['test-suite-content']],
              'child' => $container
            ]
          ]
        ]
      ]
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    foreach($this->_loadedFiles as $module => $statement) {
      if($form_state->hasValue($module) && $form_state->getValue($module) == TRUE) {
          $request = $this->getRequest();
          if($form_state->hasValue('directory')) $request->getSession()->set('directory', $form_state->getValue('directory'));
          if($form_state->hasValue('module')) $request->getSession()->set('module', $form_state->getValue('module'));
          if($form_state->hasValue('area')) $request->getSession()->set('area', $form_state->getValue('area'));
          $form_state->setRedirect('testsuite.phpunit_report');
      }
    }
  }
}
