<?php

namespace Drupal\testsuite;

use Drupal\testsuite\LoadResourceService;
use Drupal\Core\Link;
use \Drupal\Core\Url;
use Drupal\Core\Render\Markup;

/**
 * Test Suite Services.
 */
class TestMenuService {

    /**
     * @var \Drupal\testsuite\LoadResourceService
     */
    protected $_loadResourceService;

    /**
     * @param \Drupal\testsuite\LoadResourceService $loadResourceService
     */
    public function __construct(LoadResourceService $loadResourceService) {
        $this->_loadResourceService = $loadResourceService;
    }

    /**
     * Creates the top two lonks PHPUnit and Custom.
     *
     * @return array
     *  The render array of links.
     */
    public function buildTestsMenu() {
        $customUrl = Url::fromRoute('testsuite.custom');
        $phpunitUrl = Url::fromRoute('testsuite.phpunit', ['area' => 'contrib', 'module' => 'testsuite']);

        $custom_link = Link::fromTextAndUrl(t('Custom'), $customUrl);
        $phpunit_link = Link::fromTextAndUrl(t('PHPUnit'), $phpunitUrl);

        $list_items = array(
            $phpunit_link,
            $custom_link
        );

        $output['page_menu'] = [
            '#theme' => 'item_list',
            '#type' => 'ul',
            '#items' => $list_items,
            '#attached' => [
                'library' => [
                    'testsuite/menu'
                ]
            ],
            '#attributes' => array('class' => 'testsuite__list'),
            '#prefix' => '<div class="test-suite-page-menu-container">',
            '#suffix' => '</div>',

        ];

        return $output;
    }

    /**
     * Changes machine_name to Machine Name.
     *
     * @param string the machine_name
     *
     * @return string
     *   The name.
     */
    private function _createName($machine_name) {
        $parts = explode("_", $machine_name);
        return ucwords(implode(" ", $parts));
    }

    /**
     * Gets the array of files to create classes with.
     *
     * @return array
     *   The array of files to create classes with.
     */
    public function buildModuleMenu() {
        $modulesArray = $this->_loadResourceService->getResource('menu', 'phpunit', 'all', 'all', 'all');
        $list_items = array();
        $all_items = array();
        $linkString = '';
        $javaScriptUrl = Url::fromUserInput('#', ['fragment' => '#']);
        foreach($modulesArray as $area => $modules) {
            $items = array();
            if(count($modules) > 0) {
                foreach($modules as $module => $testDirectory) {
                    $directories = array();

                    foreach($testDirectory as $directory) {
                        if(!in_array($directory, $all_items)) {
                            $all_items[] = $directory;
                        }
                        $link = Link::fromTextAndUrl(
                            t($directory),
                            Url::fromRoute(
                                'testsuite.phpunit',
                                [
                                    'area' => $area,
                                    'module' => $module,
                                    'directory' => $directory
                                ]
                            )
                        )->toRenderable();
                        $link['#attributes'] = [
                            'class' => ['test-suite-directory-links']
                        ];
                        $directories[] = $link;
                    }
                    $items[] = [
                        '#markup' => '<a href="#" class="test-suite-module-links">'.t($this->_createName($module)).'<i class="arrow down"></i></a>',
                        'children' => $directories
                    ];
                }

                $list_items[] = [
                    '#markup' => '<a href="#" class="test-suite-area-links">'.ucwords($area).'</a>',
                    'children' => $items
                ];
            }
        }

        $output['modules_menu'] = [
            '#theme' => 'item_list',
            '#type' => 'ul',
            '#attributes' => array('class' => 'testsuite__list'),
            '#items' => $list_items,
            '#prefix' => '<div class="test-suite-module-menu-container">',
            '#suffix' => '</div>',
        ];

        return $output;
    }
}
